/*
 * 
 * gsasm.c - This is the main file, used to read in options and file names
 *
 * Kris Bahnsen  <kbahnsen@gmail.com>
 * Copyright (C) 2007 Kris Bahnsen
 * 
 *  Large portions of the programming routines are taken from gSXprog 
 *  Copyight (C) Eric Smith and Loren Blaney, supporing code from Jim Garlick
 * 
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 */


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <termios.h>


#include "gsasm.h"
#include "sxkey.h"
#include "sxkey_old.h"
#include "sxkey_new.h"
#include "hex_file.h"

#define GETOPT(ac,av,opt,lopt) getopt(ac,av,opt)



//#define GETOPT(ac,av,opt,lopt) getopt_long(ac,av,opt,lopt,NULL)
#define TRUE 1
#define FALSE 0

#define CMD_NONE 0x0000
#define CMD_READ 0x0001
#define CMD_ERASE 0x0002
#define CMD_PROG 0x0004
#define CMD_VERIFY 0x0008
#define CMD_RESET 0x0010
#define CMD_CLOCKON 0x0020
#define CMD_CLOCKOFF 0x0040
#define CMD_FUSEINFO 0x0080
#define CMD_ASM 0x0100



int main(int argc, char** argv)
{
	asm_file_in = hex_file_out = NULL;
	int c, cmd, cnt, trim = 0, fuse_file = 0;
	
	sxkey_t key = SXKEY_T_INIT;
	sx_t r_sx, w_sx;

	cmd = CMD_NONE;

	
	if(strlen(argv[(argc - 1)]) > 64)
	{
		printf("Error: Filename greater than 64 chars, please shorten it\n");
		exit(1);
	}
	cnt = 1;
	while ((c = getopt(argc, argv,"Rhapdrfte")) != EOF)
	{
		
		switch(c)
		{
			case 'h':
				printusage();
				break;

			case 'a':
				if(cmd != CMD_NONE) printusage();
				if(argv[cnt + 1] == 0x00 || argv[cnt + 1][0] == '-') 
				{
					fprintf(stderr,"Error: -a requires hex/source file name\n");
					exit(1);
				}
				asm_file_in = hex_file_out = argv[cnt + 1];
				trim = read_asm((char *)asm_file_in);
				
				fcloseall();
				cnt += 2; 
				
				break;

			case 'p':
				if(cmd != CMD_NONE) printusage();
				if(argv[cnt + 1] == 0x00 || argv[cnt + 1][0] == '-') 
				{
					fprintf(stderr,"Error: -p requires hex/source file name\n");
					exit(1);
				}
				asm_file_in = hex_file_out = argv[cnt + 1];
				if(strncmp(hex_file_out + (strlen(hex_file_out) - 4), ".hex", 4))
				{
					trim = read_asm((char *)asm_file_in);
					fcloseall();
				}
				cmd = CMD_PROG;
				cnt += 2; 
				break;

			case 'd':
				if(argv[cnt + 1] == 0x00 || argv[cnt + 1][0] == '-') 
				{
					fprintf(stderr,"Error: -d requires device name\n");
					exit(1);
				}
				
				key.device = argv[cnt + 1];
				
				
				cnt += 2; 
				break;
				
			case 'r':
				if(cmd != CMD_NONE) printusage();
				if(argv[cnt + 1] != 0x00 && argv[cnt + 1][0] != '-') 
				{
					asm_file_in = hex_file_out = argv[cnt + 1];
				}
				cmd = CMD_READ;
				if(hex_file_out != 0x00) cnt += 2; 
				else cnt++;
				break;
				
			case 'f':
				if(cmd != CMD_NONE) printusage();
				if(argv[cnt + 1] != 0x00 && argv[cnt + 1][0] != '-') 
				{
					asm_file_in = hex_file_out = argv[cnt + 1];
					fuse_file = 1;
				}
				cmd = CMD_FUSEINFO;
				if(hex_file_out != 0x00)
				{
					cnt += 2; 
				}
				else cnt++;
				break;
				
			case 't':
				trim = 1;
				cnt++;
				break;				
				
			case 'R':
				cmd = CMD_RESET;
				cnt++;
				break;
				
			case 'e':
				cmd = CMD_ERASE;
				cnt++;
				break;
				
			default:
				
				printusage();
				break;
		}
	}
	if(argc == 1) printusage();
	
	if(cmd)
	{
		if(!fuse_file)
		{	
			if(!new_init(key.device, B57600, &key, &w_sx) && !new_init(key.device, B19200, &key, &w_sx) && !old_init(key.device, B19200, &key))
			{
				fprintf(stderr, "GSASM: failed to establish communication with SX-Key\n");
				exit(1);
			}
		}
		
		if(!(cmd & CMD_PROG) && (trim == 1) )
		{
			fprintf(stderr,"Error: -t flag is only used when programming\n");
			exit(1);
		}
		
		
		if(cmd & CMD_PROG)
		{
			c = read_hex_file(hex_file_out, &w_sx);
			if(c)
			{
				c = key.cmd_prog(&key, &w_sx, trim, false);
		
			}
		}
		else if(cmd & CMD_READ)
		{		
			c = key.cmd_read(&key, &r_sx);
			if(c)
			{
	            if(hex_file_out) c = write_hex_file(hex_file_out, &r_sx);
	            else sx_show(&r_sx);
			}
        }
        else if(cmd & CMD_FUSEINFO)
        {
			if(hex_file_out)
			{
				c = read_hex_file(hex_file_out, &r_sx);
			}
			else if(key.cmd_read_fuses)
			{
				c = key.cmd_read_fuses(&key, &r_sx);
			}
			else
            {
            	c = key.cmd_read(&key, &r_sx);
            }
        	if (c)
        	{
            	sx_show_fuse_bits(&r_sx);
        	}
        }
        else if(cmd & CMD_RESET)
        {
        	if (key.cmd_reset)
        	{
				c = key.cmd_reset(&key);
        	}
			else 
			{
				fprintf(stderr, "GSASM: reset not implemented\n");
				exit(1);
			}
			if(!c)
			{
				fprintf(stderr, "Error Resetting device\n");
			}
        }
        else if(cmd & CMD_ERASE)
        {
        	
        	c = key.cmd_erase(&key);
        	if(!c)
        	{
				fprintf(stderr, "Error Erasing device\n");
			}
        }
        
        if(!fuse_file)
		{
			key.fini(&key);
		}
	}
        
	
	

	return 0;
}

int printusage(void)
{
	fprintf(stderr, "Usage: gsasm [-d tty] [-a|-p|-f] [-R]\n");
	fprintf(stderr, "-a source	Assemble source file, outputs source.hex\n");
	fprintf(stderr, "-p hex|source	Program SX chip with hex file or source file, assembles source\n");
	fprintf(stderr, "-r [hexfile]	Read program on SX device, optionally to a file\n");
	fprintf(stderr, "-f [hexfile]	Read and display current FUSE settings\n");
	fprintf(stderr, "-t		Calibrate internal OSC (only needed when -p is given hex file)\n");
	fprintf(stderr, "-d tty		Specify device (default /dev/ttyS0)\n");
	fprintf(stderr, "-R		Reset SX device\n");
	fprintf(stderr, "-h		This help explanation\n");
	exit(1);
}


	


