#define sxkey_h_included 1
/*
 * sxkey.h, part of gSXprog
 *
 * Copyright 2001-2003 Eric Smith <eric@brouhaha.com>
 *
 * $Id: sxkey.h,v 1.5 2003/11/18 13:59:42 garlick Exp $
 */

#define SX_52_MODE(dev)		(((dev) & 0xfff) == 2)  /* SX48/52 */
#define SX_28_MODE(dev)		(((dev) & 0xfff) != 2)  /* SX18/28 */

#define SXKEY_TYPE_NEW 		0       /* SX-Key Rev. E, Rev. F, SX-Blitz */
#define SXKEY_TYPE_OLD 		1       /* SX-Key Rev. C */

#define SX_MAX_ROM_SIZE 	4096
#define SX_ID_SIZE 		16

/* hex file offsets */
#define ID_LOC 			0x1000
#define FUSE_LOC 		0x1010
#define FUSEX_LOC 		0x1011

/* fusex bits for trimming internal RCA oscillator (modified if --trim) */
#define IRCTRIM0            0x100 
#define IRCTRIM1            0x200
#define IRCTRIM2            0x800

/* --clockon limits */
#define MIN_CLOCK   0.4
#define MAX_CLOCK   120.
/* 20 MHz reference clock */
#define REF_CLOCK   20.

#define SXKEY_ECHO_TIMEOUT 	100

#define SXKEY_DEFAULT_DEVICE    "/dev/ttyS0"

#define ARRAY_SIZE 		(SX_MAX_ROM_SIZE + SX_ID_SIZE + 2 + 256)
typedef struct {
    u16 device;
    u16 rom[ARRAY_SIZE];
} sx_t;

#define SXKEY_T_INIT {              \
    .cmd_read = NULL,               \
    .cmd_read_fuses = NULL,         \
    .cmd_prog = NULL,               \
    .cmd_erase = NULL,              \
    .cmd_clock_on = NULL,           \
    .cmd_clock_off = NULL,          \
    .cmd_reset = NULL,              \
    .fini = NULL,                   \
    .fw_rev = -1,                   \
    .fd = -1,                       \
    .speed = 0,                     \
    .device = SXKEY_DEFAULT_DEVICE, \
}

typedef struct _sxkey {
    boolean(*cmd_read) (struct _sxkey *, sx_t *);
    boolean(*cmd_read_fuses) (struct _sxkey *, sx_t *);
    boolean(*cmd_prog) (struct _sxkey *, sx_t *, boolean trim, boolean idskip);
    boolean(*cmd_erase) (struct _sxkey *);
    boolean(*cmd_clock_on) (struct _sxkey *, float);
    boolean(*cmd_clock_off) (struct _sxkey *);
    boolean(*cmd_reset) (struct _sxkey *);
    void (*fini) (struct _sxkey *);
    u8 fw_rev;
    int fd;
    speed_t speed;
    char *device;
} sxkey_t;

boolean sx_serial_open(char *device_name, speed_t bit_rate, int *fd);
void sx_serial_close(int fd);

boolean sx_serial_read(int fd, int ms_timeout, u8 * c);
boolean sx_serial_write(int fd, u8 c);

boolean sx_serial_flush(int fd);

void sx_ms_delay(int ms);

boolean sx_expect_str(int fd, int ms_timeout, char *s);
boolean sx_get_word(int fd, int ms_timeout, u16 * w);
boolean sx_get_byte(int fd, int ms_timeout, u8 * c);

boolean sx_send_str(int fd, char *s);
boolean sx_send_word(int fd, u16 w);
boolean sx_send_byte(int fd, u8 c);

void sx_setkeys(u8 c1, u8 c2);
void sx_unsetkeys(void);
int sx_rom_size(sx_t * sx);
void sx_show_fuse_bits(sx_t * sx);
void sx_show(sx_t * sx);
boolean sx_compare(sx_t * sx1, sx_t * sx2, boolean ignoretrim, boolean idskip);

/*
 * em:Local Variables: 
 * em:indent-tabs-mode:nil
 * em:c-basic-offset:4
 * em:End:
 * vi:tabstop=4 shiftwidth=4 expandtab
 */
